Package: RolDE
Type: Package
Title: RolDE: Robust longitudinal Differential Expression
Version: 1.17.0
Description: RolDE detects longitudinal differential expression between 
    two conditions in noisy high-troughput data. Suitable even for 
    data with a moderate amount of missing values.RolDE is a composite method, 
    consisting of three independent modules with different approaches
    to detecting longitudinal differential expression. The combination of 
    these diverse modules allows RolDE to robustly detect
    varying differences in longitudinal trends and expression levels in
    diverse data types and experimental settings.
Authors@R: c(person("Tommi", "Valikangas", email = "tsvali@utu.fi", role=c("aut")),
    person("Medical Bioinformatics Centre", email = "mbc@utu.fi", role=c("cre")))
License: GPL-3
Imports: stats, methods, ROTS, matrixStats, foreach, parallel,
        doParallel, doRNG, rngtools, SummarizedExperiment, nlme,
        qvalue, grDevices, graphics, utils
Depends: R (>= 4.2.0)
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.1.2
Suggests: knitr, printr, rmarkdown, testthat
VignetteBuilder: knitr
biocViews: StatisticalMethod, Software, TimeCourse, Regression,
        Proteomics, DifferentialExpression
NeedsCompilation: no
URL: https://github.com/elolab/RolDE
BugReports: https://github.com/elolab/RolDE/issues
git_url: https://git.bioconductor.org/packages/RolDE
git_branch: devel
git_last_commit: 6cb2245
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-29 00:45:42 UTC; biocbuild
Author: Tommi Valikangas [aut],
  Medical Bioinformatics Centre [cre]
Maintainer: Medical Bioinformatics Centre <mbc@utu.fi>
