# Generated by roxygen2: do not edit by hand

export(RolDE)
export(RolDE_Main)
export(plotFindings)
import(SummarizedExperiment)
import(doRNG)
import(rngtools)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(methods,is)
importFrom(nlme,lmeControl)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(utils,head)
importFrom(utils,tail)
