#' A RolDE design matrix for data1
#'
#' A design matrix to be used with data1 for the RolDE-function. Column 1 contains sample names,
#' column 2 the condition information for each sample, column 3 indicates the timepoint for each sample,
#' and column 4 gives the Replicate or Individual each sample is coming from.
#' @format A matrix with 30 rows and 4 variables.
"des_matrix1"
