#' A proteomics dataset with random protein expression values - no missing values.
#'
#' A longitudinal proteomics dataset with five timepoints, two conditions and
#' three replicates for each sample at each timepoint in each condition. The expression
#' values for the random data have been generated using the \code{rnorm} function.
#' The values for each protein (row) have been drawn from a normal distribution with a
#' mean of 22.5 and a standard deviation of 1.5 corresponding to random protein expression
#' values.
#'
#' @format A matrix with 1045 rows and 30 variables.
"data1"
