#' A semi-simulated UPS1 spike-in dataset with differences in longitudinal expression for the spike-in proteins - no missing values.
#'
#' A longitudinal proteomics dataset with five timepoints, two conditions and
#' three replicates for each sample at each timepoint in each condition. The expression
#' values for each protein in each sample have been generated by using the \code{rnorm}
#' function and the means and standard deviations of the corresponding proteins and samples
#' in the original experimental UPS1 spike-in data. In this manner, the expression for each
#' protein in each sample could be replicated with some random variation multiple times when
#' necessary. The pattern of missing values was directly copied from the used samples
#' in the original UPS1 spike-in dataset. All proteins with missing values have been filtered out.
#' Linear-like trends for the spike-in proteins for Condition 1 were generated using the 4,4,10,25 and 50 fmol samples of the original
#' UPS1 spike-in data. Linear-like trends for the spike-in proteins for Condition 2 were generated using the 2,4,4,10 and 25 fmol samples
#' of the original UPS1 spike-in data. For more information about the generation of the semi-simulated spike-in datasets,
#' see the original RolDE publication. In the spike-in datasets, the UPS1
#' spike-in proteins are expected to differ between conditions while the expression of the rest
#' of the proteins (the background proteins) are expected to remain stable between the conditions,
#' excluding experimental noise.
#'
#' @format A matrix with 1033 rows and 30 variables.
#' @source \url{https://www.ebi.ac.uk/pride/archive/projects/PXD002099}
"data3"
